"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TLRU = void 0;
const fun_dispatcher_1 = require("fun-dispatcher");
class TLRU extends Map {
    constructor(options = {}) {
        super();
        this.maxStoreSize = 1000;
        this.maxAgeMs = 12 * 60 * 60 * 1000;
        this.scheduler = new fun_dispatcher_1.Scheduler();
        this.defaultLRU = false;
        if (options.maxAgeMs)
            this.maxAgeMs = options.maxAgeMs;
        if (options.maxStoreSize)
            this.maxStoreSize = options.maxStoreSize;
        if (options.defaultLRU)
            this.defaultLRU = true;
        if (options.disposer)
            this.disposer = options.disposer;
    }
    set(key, value, ttuMs = this.maxAgeMs + this.size) {
        if (this.size >= this.maxStoreSize) {
            this.scheduler.runNext();
        }
        this.scheduler.schedule(String(key), () => {
            if (typeof this.disposer === "function" && super.has(key))
                this.disposer(super.get(key));
            super.delete(key);
        }, ttuMs);
        return super.set(key, value);
    }
    get(key, revive = this.defaultLRU) {
        if (this.has(key)) {
            if (revive) {
                const original = this.scheduler.get(String(key));
                if (original)
                    this.scheduler.schedule(String(key), () => super.delete(key), original.delay + super.size);
            }
            return super.get(key);
        }
        return undefined;
    }
    delete(key) {
        this.scheduler.delete(String(key));
        return super.delete(key);
    }
    clear() {
        this.scheduler.flush();
        super.clear();
    }
    toJSON() {
        return [...this].sort(([v], [v1]) => {
            const t1 = this.scheduler.get(String(v));
            const t2 = this.scheduler.get(String(v1));
            if (!t1)
                return -1;
            if (!t2)
                return 1;
            return t1.expiry - t2.expiry;
        });
    }
}
exports.TLRU = TLRU;
//# sourceMappingURL=index.js.map